`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer:		 D.A. Heitman 
// 
// Create Date:    21:15:46 02/14/2011 
// Design Name: 
// Module Name:    Trigger 
// Project Name: 
// Target Devices: Xylo-lm
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
/*
	Copyright: D.A. Heitman
    This file is part of a simple logic analyzer.

    This logic analyzer is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    logic analyzer is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this logic analyzer.  If not, see <http://www.gnu.org/licenses/>.
*/
module Trigger(
    input mClk,
    input Reset,
    input SetMask12,
    input SetMask34,
    input SetValue12,
    input SetValue34,
	 input SetStart,
    input [31:0] Data,
    input Arm,
    output reg Armed,
    output reg Fired,
	 input[15:0] Channel,
	 output[5:0] Status
    );
reg[15:0]Channel_prev=0;
reg[15:0]TriggerMask1=0,TriggerMask1_next=0;
reg[15:0]TriggerMask2=0,TriggerMask2_next=0;
reg[15:0]TriggerMask3=0,TriggerMask3_next=0;
reg[15:0]TriggerMask4=0,TriggerMask4_next=0;
reg[15:0]TriggerValue1=0,TriggerValue1_next=0;
reg[15:0]TriggerValue2=0,TriggerValue2_next=0;
reg[15:0]TriggerValue3=0,TriggerValue3_next=0;
reg[15:0]TriggerValue4=0,TriggerValue4_next=0;
reg[2:0] TriggerStart=4'b111,TriggerStart_next;
reg[3:0] TriggerAny=4'b0000,TriggerAny_next=4'b0000;
reg[3:0] trgState=0,trgState_next=0;
reg Armed_next,Fired_next;

//assign Status=TriggerMask1[5:0];
//assign Status=TriggerValue1[5:0];
//assign Status = 6'b010101;
assign Status={2'b0,1'b0,Armed,Fired,1'b0};
always @(posedge mClk) begin
	if(Reset) trgState<=0;
	else
		trgState<=trgState_next;
		TriggerMask1<=TriggerMask1_next;
		TriggerMask2<=TriggerMask2_next;
		TriggerMask3<=TriggerMask3_next;
		TriggerMask4<=TriggerMask4_next;
		TriggerValue1<=TriggerValue1_next;
		TriggerValue2<=TriggerValue2_next;
		TriggerValue3<=TriggerValue3_next;
		TriggerValue4<=TriggerValue4_next;
		TriggerStart<=TriggerStart_next;
		TriggerAny<=TriggerAny_next;
		Armed<=Armed_next;
		Fired<=Fired_next;
	end
//
localparam INITIALWAIT 	= 0;
localparam ARMED 			= 1;
localparam ENDTRIGGER1 	= 2;
localparam TRIGGER2 		= 3;
localparam ENDTRIGGER2 	= 4;
localparam TRIGGER3 		= 5;
localparam ENDTRIGGER3 	= 6;
localparam TRIGGER4 		= 7;
localparam ALLFIRED 		= 8;

wire any1=TriggerAny[0]&& ((Channel_prev[15:0]&TriggerMask1[15:0])!=(Channel[15:0]&TriggerMask1[15:0]));
wire any2=TriggerAny[1]&& ((Channel_prev[15:0]&TriggerMask2[15:0])!=(Channel[15:0]&TriggerMask2[15:0]));
wire any3=TriggerAny[2]&& ((Channel_prev[15:0]&TriggerMask3[15:0])!=(Channel[15:0]&TriggerMask3[15:0]));
wire any4=TriggerAny[3]&& ((Channel_prev[15:0]&TriggerMask4[15:0])!=(Channel[15:0]&TriggerMask4[15:0]));
wire trg1=(Channel[15:0]&TriggerMask1[15:0])==(TriggerValue1[15:0]&TriggerMask1[15:0]);
wire trg2=(Channel[15:0]&TriggerMask2[15:0])==(TriggerValue2[15:0]&TriggerMask2[15:0]);
wire trg3=(Channel[15:0]&TriggerMask3[15:0])==(TriggerValue3[15:0]&TriggerMask3[15:0]);
wire trg4=(Channel[15:0]&TriggerMask4[15:0])==(TriggerValue4[15:0]&TriggerMask4[15:0]);
always @* begin
	trgState_next=trgState;
	TriggerMask1_next=SetMask12?Data[15:0]:TriggerMask1;
	TriggerMask2_next=SetMask12?Data[31:16]:TriggerMask2;
	TriggerMask3_next=SetMask34?Data[15:0]:TriggerMask3;
	TriggerMask4_next=SetMask34?Data[31:16]:TriggerMask4;
	TriggerValue1_next=SetValue12?Data[15:0]:TriggerValue1;
	TriggerValue2_next=SetValue12?Data[31:16]:TriggerValue2;
	TriggerValue3_next=SetValue34?Data[15:0]:TriggerValue3;
	TriggerValue4_next=SetValue34?Data[31:16]:TriggerValue4;
	TriggerStart_next=SetStart?Data[2:0]:TriggerStart;
	TriggerAny_next=SetStart?Data[7:4]:TriggerAny;
	Channel_prev=SetStart?Channel:Channel_prev;
	Armed_next=0;
	Fired_next=0;
	case(trgState)
	INITIALWAIT: if(Arm) begin 
		trgState_next=ARMED;
		Armed_next=1'b1;
		end
	ARMED: begin
		Armed_next=1'b1;
		if(any1 || trg1) begin
			if(TriggerStart[0]==1'b1) begin
				trgState_next=ALLFIRED;
				Fired_next=1'b1;
				end
			else trgState_next=ENDTRIGGER1;
			end
		end
	ENDTRIGGER1: begin // wait for trigger 1 to end
		Armed_next=1'b1;
		if(~trg1) begin 
			trgState_next=TRIGGER2;
			Channel_prev=Channel;
			end
		end
	TRIGGER2: begin
		Armed_next=1'b1;
		if(any2 || trg2) begin
			if(TriggerStart[1]==1'b1) begin
				trgState_next=ALLFIRED;
				Fired_next=1'b1;
				end
			else trgState_next=ENDTRIGGER2;
			end
		end
	ENDTRIGGER2: begin // wait for trigger 2 to end
		Armed_next=1'b1;
		if(~trg2) begin
			trgState_next=TRIGGER3;
			Channel_prev=Channel;
			end
		end
	TRIGGER3: begin
		Armed_next=1'b1;
		if(any3 || trg3) begin
			if(TriggerStart[2]==1'b1) begin
				trgState_next=ALLFIRED;
				Fired_next=1'b1;
				end
			else trgState_next=ENDTRIGGER3;
			end
		end
	ENDTRIGGER3: begin // wait for trigger 3 to end
		Armed_next=1'b1;
		if(~trg3) begin
			trgState_next=TRIGGER4;
			Channel_prev=Channel;
			end
		end
	TRIGGER4: begin
		Armed_next=1'b1;
		if(any4 || trg4) begin
			trgState_next=ALLFIRED;
			Fired_next=1'b1;
			end
		end
	ALLFIRED: begin
		if(!Arm)
			Fired_next=1'b1;
		else
			trgState_next=ARMED;
		end
	endcase
	end
//
endmodule
