`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 		 D.A. Heitman
// 
// Create Date:    20:11:35 02/14/2011 
// Design Name: 
// Module Name:    UsbCmd 
// Project Name: 
// Target Devices: Xylo-lm
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
/*
	Copyright: D.A. Heitman
    This file is part of a simple logic analyzer.

    This logic analyzer is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    logic analyzer is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this logic analyzer.  If not, see <http://www.gnu.org/licenses/>.
*/
module UsbCmd(
    input FX2_CLK,
    inout [7:0] FX2_FD,
    output FX2_SLRD,
    output FX2_SLWR,
    input [0:0] FX2_FLAGS,
    output FX2_PA_2,
    output FX2_PA_3,
    output FX2_PA_4,
    output FX2_PA_5,
    output FX2_PA_6,
//    input FX2_PA_7,

    input mClk,
    input Reset,
	 output reg Execute,
    output reg[7:0] Cmd,
    output reg[31:0] Data,
	 output reg[13:0] Mem_Addr,
	 input [15:0] Mem_Data,
	 input [15:0] LaStatus,
	 output [5:0]UsbStatus
    );
////////////////////////////////////////////////////////////////////////////////
// Rename "FX2" ports into "FIFO" ports, to give them more meaningful names
// FX2 USB signals are active low, take care of them now
// Note: You probably don't need to change anything in this section

// FX2 outputs
wire FIFO_CLK = FX2_CLK;

wire FIFO2_empty = ~FX2_FLAGS[0];	wire FIFO2_data_available = ~FIFO2_empty;
//wire FIFO3_empty = ~FX2_FLAGS[1];	wire FIFO3_data_available = ~FIFO3_empty;
//wire FIFO4_full = ~FX2_FLAGS[2];	wire FIFO4_ready_to_accept_data = ~FIFO4_full;
//wire FIFO5_full = ~FX2_PA_7;		wire FIFO5_ready_to_accept_data = ~FIFO5_full;
//assign FX2_PA_0 = 1'b1;
//assign FX2_PA_1 = 1'b1;
assign FX2_PA_3 = 1'b1;

// FX2 inputs
wire FIFO_RD, FIFO_WR, FIFO_PKTEND, FIFO_DATAIN_OE, FIFO_DATAOUT_OE;
assign FX2_SLRD = ~FIFO_RD;
assign FX2_SLWR = ~FIFO_WR;
assign FX2_PA_2 = ~FIFO_DATAIN_OE;
assign FX2_PA_6 = ~FIFO_PKTEND;

wire [1:0] FIFO_FIFOADR;
assign {FX2_PA_5, FX2_PA_4} = FIFO_FIFOADR;

// FX2 bidirectional data bus
wire [7:0] FIFO_DATAIN = FX2_FD;
wire [7:0] FIFO_DATAOUT;
reg[7:0]FIFO_DATAOUT_reg;
assign FX2_FD = FIFO_DATAOUT_OE ? FIFO_DATAOUT : 8'hZZ;
////////////////////////////////////////////////////////////////////////////////
// So now everything is in positive logic
//	FIFO_RD, FIFO_WR, FIFO_DATAIN, FIFO_DATAOUT, FIFO_DATAIN_OE, FIFO_DATAOUT_OE, FIFO_PKTEND, FIFO_FIFOADR
//	FIFO2_empty, FIFO2_data_available
//	FIFO3_empty, FIFO3_data_available
//	FIFO4_full, FIFO4_ready_to_accept_data
//	FIFO5_full, FIFO5_ready_to_accept_data

//assign FIFO_FIFOADR = 2'b00;		// select FIFO2
reg[1:0] FIFO_FIFOADR_reg;
assign FIFO_FIFOADR = FIFO_FIFOADR_reg;
//assign FIFO_RD = 1'b1;			// always read
//assign FIFO_WR = 1'b0;			// never write
//assign FIFO_DATAOUT = 8'h00;		// never write, so this value is not used
//assign FIFO_DATAIN_OE = 1'b1;	// always read data
//assign FIFO_DATAOUT_OE = 1'b0;	// never output data
//assign FIFO_PKTEND = 1'b0;

localparam IDLE = 0;
localparam READCMD = 1; 
localparam READADR1 = 2;
localparam READADR2 = 3;
localparam READADR3 = 4; 
localparam READADR4 = 5; 
localparam READLEN1 = 6; 
localparam READLEN2 = 7; 
localparam EXECCMD = 8; 
localparam READMEM = 9; 
localparam READMEM2 = 10; 
localparam READSTATUS=11;
localparam READSTATUS2=12;
localparam READEND = 13;
localparam WAIT1CLK = 14;
localparam WAIT1CLK2 = 15;
reg[4:0] USB_State = IDLE,USB_State_next=IDLE;
reg[11:0] Len;
reg Len1k=0;
assign UsbStatus=Cmd;
// FSM
always @(posedge FIFO_CLK or posedge Reset)
	if(Reset) begin
		USB_State<=IDLE;
		end
	else USB_State<=USB_State_next;
// FSM Combinational next state logic
always @* begin
	FIFO_FIFOADR_reg=2'b00;
	USB_State_next = USB_State;
	FIFO_FIFOADR_reg=2'b00;
	case(USB_State)
	IDLE: if(FIFO2_data_available) USB_State_next = READCMD;
		READCMD:  USB_State_next = READADR1; 
		READADR1: USB_State_next = READADR2; 
		READADR2: USB_State_next = READADR3;
		READADR3: USB_State_next = READADR4;
		READADR4: USB_State_next = READLEN1;
		READLEN1: USB_State_next = READLEN2;
		READLEN2: USB_State_next = EXECCMD;
		EXECCMD: begin
			case(Cmd)
			default:USB_State_next = IDLE;
			1: USB_State_next = WAIT1CLK;
			2: USB_State_next = READSTATUS;
			endcase
			end
		WAIT1CLK:
			USB_State_next=WAIT1CLK2;
		WAIT1CLK2:
			USB_State_next=READMEM;
		READMEM:begin
			FIFO_FIFOADR_reg=2'b10;
			if(Len==1) USB_State_next=READEND;
			else USB_State_next=READMEM2;
			end
		READMEM2:begin
			FIFO_FIFOADR_reg=2'b10;
			if(Len==1) USB_State_next=READEND; // should never happen
			else USB_State_next=WAIT1CLK2;
			end
		READEND: begin
			FIFO_FIFOADR_reg=2'b10;
			USB_State_next=IDLE;
			end
		READSTATUS: begin
			FIFO_FIFOADR_reg=2'b10;
			USB_State_next=READSTATUS2;
			end
		READSTATUS2: begin
			FIFO_FIFOADR_reg=2'b10;
			USB_State_next=READEND;
			end
	endcase
	end
// Data path parts
// clock the data from the PC into the right register
reg Execute_cmd=0,Execute_FF=0;
always @(posedge FIFO_CLK)
	case(USB_State)
		IDLE: Execute_cmd<=1'b0;
		READCMD:  Cmd<=FIFO_DATAIN;
		READADR1: Data[7:0]<= FIFO_DATAIN;
		READADR2: Data[15:8]<= FIFO_DATAIN;
		READADR3: Data[23:16]<= FIFO_DATAIN;
		READADR4: Data[31:24]<= FIFO_DATAIN;
		READLEN1: Len[7:0]<= FIFO_DATAIN;
		READLEN2: begin 
			Len[11:8]<= FIFO_DATAIN[3:0];
			Len1k<=({FIFO_DATAIN[3:0],Len[7:0]}==1024);
			end
		EXECCMD: begin Execute_cmd<=1'b1;Mem_Addr<=Data;end
		READMEM:	begin 
			Len<=Len-1;
			Mem_Addr<=Mem_Addr+1;
			end
		READMEM2:	begin
			Len<=Len-1;
			end
		READSTATUS2: begin
			Len1k<=0;
			Len<=0;
			end
	endcase
// FIFO read write signals
assign FIFO_RD = (USB_State==READCMD)||
					(USB_State==READADR1)||(USB_State==READADR2)||(USB_State==READADR3)||(USB_State==READADR4)||
					(USB_State==READLEN1)||(USB_State==READLEN2);
assign FIFO_DATAIN_OE = FIFO_RD;
assign FIFO_WR = (USB_State ==READMEM) || (USB_State==READMEM2) || (USB_State==READSTATUS)||(USB_State==READSTATUS2);
assign FIFO_DATAOUT_OE = FIFO_WR;
assign FIFO_DATAOUT = FIFO_DATAOUT_reg;
assign FIFO_PKTEND = (USB_State==READEND)&&(!Len1k);
// FIFOout data clocking
// Clock the data into FIFO register at the right moment
reg [15:0]MemWord;
always @(negedge FIFO_CLK)
	case(USB_State)
	WAIT1CLK2:		MemWord<=Mem_Data;
	READMEM:			FIFO_DATAOUT_reg<=MemWord[7:0];
	READMEM2:		FIFO_DATAOUT_reg<=MemWord[15:8];
	
	READSTATUS:		FIFO_DATAOUT_reg<=LaStatus[7:0];
	READSTATUS2:	FIFO_DATAOUT_reg<=LaStatus[15:8];
	endcase
// synchronize the execute signal with the main clock	(must be higher freq then FX2_CLK)
always @(posedge mClk)
	if(Execute_cmd) begin
		if(Execute_FF==0) begin
			Execute<=1'b1;
			Execute_FF<=1'b1;
			end
		else
			Execute<=1'b0;
		end
	else begin
		Execute<=1'b0;
		Execute_FF<=1'b0;
		end
endmodule
