// LogicAnalyzerView.cpp : implementation of the CLogicAnalyzerView class
//
/////////////////////////////////////////////////////////////////////////////
/*
	Copyright: D.A. Heitman
    This file is part of a simple logic analyzer.

    This logic analyzer is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    logic analyzer is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this logic analyzer.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "stdafx.h"
#include "resource.h"

#include "LogicAnalyzerView.h"
#include "SetTrigger.h"
CLogicAnalyzerView::CLogicAnalyzerView()
{
	for(int i=0;i<4;++i)
	{
		Mask[i]=1;
		Value[i]=1;
		StartSampling[i]=true;
	}
	PreAmble=8;
	Clock=2;
}

BOOL CLogicAnalyzerView::PreTranslateMessage(MSG* pMsg)
{
	pMsg;
	return FALSE;
}
LRESULT CLogicAnalyzerView::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	USBDevice = new CCyUSBDevice(NULL, GUID_KNJN_FX2);
	rDown=lDown=false;
	Start=0;Width=1024;
	memset(buffer,0,sizeof(buffer));
	LeftCt=RightCt=UpCt=DownCt=0;

	InfoDlg.Create(m_hWnd,0);
	return 0;
}
LRESULT CLogicAnalyzerView::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	delete USBDevice;	
	return 0;
}

LRESULT CLogicAnalyzerView::OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	CRect rect;
	GetClientRect(rect);
	CRect drect;
	InfoDlg.GetClientRect(drect);
	CPaintDC dc(m_hWnd);

	Display();

	dc.SelectBrush(GetSysColorBrush(COLOR_3DFACE));
	dc.Rectangle(0,rect.bottom-drect.bottom-4,rect.right,rect.bottom);
	InfoDlg.SetWindowPos(HWND_TOP,2,rect.bottom-drect.bottom-2,drect.right,drect.bottom,SWP_SHOWWINDOW);

	return 0;
}
void CLogicAnalyzerView::Set()
{
#pragma pack(push)
#pragma pack(1)
	struct
	{
		UCHAR Cmd;
		union{
			ULONG Data;
			WORD wData[2];
			BYTE bData[4];
		};
		USHORT Len;
	}Command;

	if(USBDevice->ControlEndPt==NULL){MessageBox(_T("Usb Device/Fpga not available!"),_T("Error"),MB_OK);return;}
	int Status=0;
#pragma pack(pop)
	LONG clen=sizeof(Command);
	Command.Cmd=3; //  Reset
	BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
	Command.Cmd=4; // Set Clock
	Command.Data=Clock;
	BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
	Command.Cmd=5; // set preamble
	Command.Data=PreAmble;
	BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
	Command.Cmd=6; // set mask12
	Command.wData[0]=Mask[0];
	Command.wData[1]=Mask[1];
	BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
	Command.Cmd=7; // set mask34
	Command.wData[0]=Mask[2];
	Command.wData[1]=Mask[3];
	BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
	Command.Cmd=8;// set value12
	Command.wData[0]=Value[0];
	Command.wData[1]=Value[1];
	BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
	Command.Cmd=9; // set value34
	Command.wData[0]=Value[2];
	Command.wData[1]=Value[3];
	BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
	Command.Cmd=14;// start Sampling mask
	Command.wData[0]=StartSampling[0]+(StartSampling[1]<<1)+(StartSampling[2]<<2);
	Command.wData[0]+=(AnyChange[0]<<4) +(AnyChange[1]<<5)+(AnyChange[2]<<6)+(AnyChange[3]<<7);
	BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
	Command.Cmd=15;// Arm
	BulkOutPipe2->XferData((PUCHAR)&Command, clen);	

	do{
		Command.Cmd=2; // read status
		LONG rlenst=2;
		BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
		BulkInPipe4->XferData((PUCHAR)&Status, rlenst);
	}while((Status&0x4000) ==0); // Fired and not Started?
	while(Status&0x8000) // Still sampling?
	{
		Command.Cmd=2; // read status
		LONG rlenst=2;
		BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
		BulkInPipe4->XferData((PUCHAR)&Status, rlenst);
	}

	for(int i=0;i<sizeof(buffer)/1024;++i)
	{
		Command.Cmd=1; // read memory
		Command.Data=(Status&0x3fff) + (512*i);
		Command.Len=1024;
		BulkOutPipe2->XferData((PUCHAR)&Command, clen);	
		LONG rlen=1024;
		BulkInPipe4->XferData((i*1024)+(PUCHAR)&buffer, rlen);
	}
	int min0[16],min1[16],ct0[16],ct1[16],f[16];
	for(int i=0;i<16;++i)min0[i]=min1[i]=BUFFERSIZE+1;
	WORD w=buffer[0];
	for(int j=0;j<16;++j)f[j]=0;
	for(int j=0;j<16;++j)if(w&(1<<j)){ct0[j]=0;ct1[j]=1;}else{ct0[j]=1;ct1[j]=0;}
	// find the minimum 0 and 1 
	for(int i=1;i<BUFFERSIZE;++i)
	{
		for(int j=0;j<16;++j)
		{
			if((w^buffer[i])&(1<<j))
			{
				if(f[j]==0){f[j]=1;w=buffer[i];
					if(w&(1<<j))ct1[j]=0;else ct0[j]=0;
					continue;} // first transistion, if so ignore
				if((w&(1<<j))==0){ // transition to 1
					if(ct0[j]<min0[j])min0[j]=ct0[j];
					ct0[j]=1;
				}
				else{ // transition to 0
					if(ct1[j]<min1[j])min1[j]=ct1[j];
					ct1[j]=1;
				}
				w=buffer[i];
			}
			else{
				if(w&(1<<j))ct1[j]++;else ct0[j]++;
			}
		}
	}
	// now count the minimum 0 & 1, also count the minum size plus 1,as we could have missed the transition during the sampling
	// this is the average out and find the frequency of a possibble clock
	int pct0[16],pct1[16],pct0p1[16],pct1p1[16];
	for(int i=0;i<16;++i)pct0[i]=pct1[i]=pct0p1[i]=pct1p1[i]=0;
	for(int j=0;j<16;++j)if(w&(1<<j)){ct0[j]=0;ct1[j]=1;}else{ct0[j]=1;ct1[j]=0;}
	for(int i=1;i<BUFFERSIZE;++i)
	{
		for(int j=0;j<16;++j)
		{
			if((w^buffer[i])&(1<<j))
			{
				if((w&(1<<j))==0){ // transition to 1
					if(ct0[j]==min0[j])pct0[j]++;
					if(ct0[j]==min0[j]+1)pct0p1[j]++;
					ct0[j]=1;
				}
				else{ // transition to 0
					if(ct1[j]==min1[j])pct1[j]++;
					if(ct1[j]<=min1[j]+2)pct1p1[j]++;
					ct1[j]=1;
				}
				w=buffer[i];
			}
			else{
				if(w&(1<<j))ct1[j]++;else ct0[j]++;
			}
		}
	}
	float p0[16],p1[16],pct[16];
	for(int i=0;i<16;++i){
		if(pct0[i]+pct0p1[i]==0){pct[i]=0;continue;}
		if(pct1[i]+pct1p1[i]==0){pct[i]=0;continue;}
		p0[i]=(float)(min0[i]*pct0[i]+pct0p1[i]*(min0[i]+1))/(pct0[i]+pct0p1[i]);
		p1[i]=(float)(min1[i]*pct1[i]+pct1p1[i]*(min1[i]+1))/(pct1[i]+pct1p1[i]);
		pct[i]=(p0[i]+p1[i])/2;
	}
	for(int i=0;i<16;++i)
		if(pct[i]==0)
			InfoDlg.Mhz[i]=0;
		else
			InfoDlg.Mhz[i]=200./(2*Clock)/(2*pct[i]);

	InfoDlg.RedrawWindow(NULL,NULL,RDW_INVALIDATE);
	Display();

}

void CLogicAnalyzerView::Display()
{
	CRect rect;
	GetClientRect(rect);
	CRect drect;
	InfoDlg.GetClientRect(drect);

	CClientDC dc(m_hWnd);
	CDC cdc;
	cdc.CreateCompatibleDC(dc);
	HBITMAP bmp=CreateCompatibleBitmap(dc,rect.right-4,rect.bottom-drect.bottom-8);
	cdc.SelectBitmap(bmp);
	cdc.SelectBrush((HBRUSH)GetStockObject(BLACK_BRUSH));

	cdc.Rectangle(CRect(2,2,rect.right-4,rect.bottom-drect.bottom-4));
	HPEN bPen=CreatePen(PS_SOLID,1,RGB(0,63,0));
	cdc.SelectPen(bPen);
	for(int x=0;x<rect.right-4;x+=25)
	{
		cdc.MoveTo(x,0);
		cdc.LineTo(x,rect.bottom);
	}
	for(int y=0;y<rect.bottom-drect.bottom-8;y+=25)
	{
		cdc.MoveTo(0,y);
		cdc.LineTo(rect.right,y);
	}
	HPEN hPen=CreatePen(PS_SOLID,1,RGB(0,255,0));
	cdc.SelectPen(hPen);
	HBRUSH bBrush=CreateSolidBrush(RGB(0,63,0));
	cdc.SelectBrush(bBrush);
//	cdc.SelectPen((HPEN)GetStockObject(WHITE_PEN));
	double delta=(double)rect.right/Width;
	double cl = 200./(2*Clock); // 200mhz/(2*Clock) = nr mhz of sampleclock (200mhz = 25*24/3 Clock from DCM)
							// this is one delta nr of screen points, which means the 25 screen points (one division)
							// is (1000/cl) *( 25/delta)ns.
	double nns=(1000/cl)*(25/delta);
	TCHAR buf[100];
	CFont cf;
	cf.CreateFont(-10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,_T("Times New Roman"));
	HFONT hfo=cdc.SelectFont(cf);
	cdc.SetBkColor(TRANSPARENT);

	if(nns<1000.)
		_stprintf(buf,_T("%3.2fns"),nns);
	else if(nns<1000000.)
		_stprintf(buf,_T("%3.2fus"),nns/1000.);
	else
		_stprintf(buf,_T("%3.2fms"),nns/1000000.);
	cdc.SetTextColor(RGB(255,255,255));
	cdc.TextOut(5,30,buf);
	cdc.SetTextColor(RGB(255,255,0));
	for(int i=0;5*i*25<rect.right;++i)
	{
		double f=5*nns/25;
		f=f*(Start+25*i);
		if(f<1000.)
			_stprintf(buf,_T("%3.2fns"),f);
		else if(f<1000000.)
			_stprintf(buf,_T("%3.2fus"),f/1000.);
		else
			_stprintf(buf,_T("%3.2fms"),f/1000000.);
		cdc.TextOut(25*5*i,5,buf);
	}

	cdc.SelectFont(hfo);



	int old[16];
	for(int j=0;j<16;++j)
		old[j]=(buffer[Start]>>j)&1;
	double xw=0;
	for(int i=0;i<Width;++i)
	{	
		double x=i*delta;
		if(delta>1)
		{
			if((i&1)==0)
			{
				cdc.MoveTo((int)x,60);
				cdc.LineTo((int)(x+delta),60);
			}
		}
		xw+=delta;
		if(xw<1)
		{
			WORD b=buffer[Start+i];
			for(int j=0;j<16;++j)
			{
				if(old[j]==0)
				{
					if((b>>j)&1)old[j]=2;
				}
				else if(old[j]==1)
					if(0==((b>>j)&1))old[j]=2;
			}
		}
		else
		{
			WORD b=buffer[Start+i];
			for(int j=0;j<16;++j)
			{
				if(!InfoDlg.Channel[j]) continue;
				WORD bold=old[j];
				WORD bnew=(b>>j)&1;
				if(bnew)
				{
					cdc.SelectPen(bPen);
					cdc.Rectangle(x,j*16-10+80,x+xw,j*16+80+1);
					cdc.SelectPen(hPen);
				}
				if((bnew!=bold))
				{
					cdc.MoveTo((int)x,j*16-10+80);
					cdc.LineTo((int)(x),j*16+80);
				}
				old[j]=bnew;

				cdc.MoveTo((int)x,j*16-bnew*10+80);
				cdc.LineTo((int)(x+xw),j*16-bnew*10+80);
			}
			xw-=(int)xw;
		}
	}
	dc.DrawEdge(CRect(0,0,rect.right,rect.bottom-drect.bottom-4),BDR_SUNKENINNER|BDR_RAISEDOUTER,BF_RECT);
	dc.BitBlt(2,2,rect.right-4,rect.bottom-drect.bottom-8,cdc,0,0,SRCCOPY);
	DeleteObject(bmp);
	DeleteObject(hPen);
	DeleteObject(bPen);
	DeleteObject(bBrush);
}
void CLogicAnalyzerView::SetTrigger()
{

	CSetTrigger dlg;
	for(int i=0;i<4;++i)
	{
		dlg.Mask[i]=Mask[i];
		dlg.Value[i]=Value[i];
		dlg.StartSampling[i]=StartSampling[i];
		dlg.AnyChange[i]=AnyChange[i];
	}
	dlg.PreAmble=PreAmble;
	dlg.Clock=Clock;
	if(dlg.DoModal())
	{
		for(int i=0;i<4;++i)
		{
			Mask[i]=dlg.Mask[i];
			Value[i]=dlg.Value[i];
			StartSampling[i]=dlg.StartSampling[i];
			AnyChange[i]=dlg.AnyChange[i];
		}
		PreAmble=dlg.PreAmble;
		Clock=dlg.Clock;

	}
}
LRESULT CLogicAnalyzerView::OnLButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	lDown=true;
	mPos.x=LOWORD(lParam);
	mPos.y=HIWORD(lParam);
	return 0;
}
LRESULT CLogicAnalyzerView::OnRButtonDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	rDown=true;
	mPos.x=LOWORD(lParam);
	mPos.y=HIWORD(lParam);
	return 0;
}
LRESULT CLogicAnalyzerView::OnLButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	lDown=false;
	return 0;
}
LRESULT CLogicAnalyzerView::OnRButtonUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	rDown=false;
	return 0;
}
LRESULT CLogicAnalyzerView::OnMouseMove(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	lDown=(wParam&MK_LBUTTON)!=0;
	rDown=(wParam&MK_RBUTTON)!=0;
	if(lDown)
	{
		Start+=2*(LOWORD(lParam)-mPos.x);
		Width+=4*(HIWORD(lParam)-mPos.y);
		if(Width<100)Width=100;
		if(Width>(BUFFERSIZE-100)) Width=BUFFERSIZE-100;
		if(Start<0)Start=0;
		if(Start>=(BUFFERSIZE-Width))Start=BUFFERSIZE-Width;
		Display();
		mPos.x=LOWORD(lParam);
		mPos.y=HIWORD(lParam);

	}
	return 0;
}
LRESULT CLogicAnalyzerView::OnEraseBackground(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{ // ignore as I don't need to and ignoring it makes a faster redraw
	return 0;
}
LRESULT CLogicAnalyzerView::OnKeyDown(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	switch(wParam)
	{
	default: bHandled=FALSE;break;
	case VK_DOWN: UpCt=LeftCt=RightCt=0;
		if(DownCt==5)
			Width-=100;
		else
		{
			Width-=10;
			DownCt++;
		}
		if(Width<100)Width=100;
		break;
	case VK_UP: DownCt=LeftCt=RightCt=0;
		if(UpCt==5)
			Width+=100;
		else
		{
			Width+=10;
			UpCt++;
		}
		if(Width>(BUFFERSIZE-Start)) 
		{
			Start=BUFFERSIZE-Width;
			if(Start<0){Start=0;Width=BUFFERSIZE;}
		}
		break;
	case VK_RIGHT: UpCt=DownCt=LeftCt=0;
		if(RightCt==5)
			Start+=100;
		else
		{
			Start++;
			RightCt++;
		}
		if(Start>=(BUFFERSIZE-Width))Start=BUFFERSIZE-Width;
		break;
	case VK_LEFT: UpCt=DownCt=RightCt=0;
		if(LeftCt==5)
			Start-=100;
		else
		{
			Start--;
			LeftCt++;
		}
		if(Start<0)Start=0;
		break;
	case VK_HOME:
		Start=0;
		break;
	case VK_END:
		Start=BUFFERSIZE-Width;
		break;
	}
	if(bHandled)
			Display();
	return 0;
}
LRESULT CLogicAnalyzerView::OnKeyUp(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	switch(wParam)
	{
	default: bHandled=FALSE;break;
	case VK_DOWN: 
		DownCt=0;
		break;
	case VK_UP: 
		UpCt=0;
		break;
	case VK_RIGHT:
		RightCt=0;
		break;
	case VK_LEFT:
		LeftCt=0;
		break;
	}
	return 0;
}
